// --------------------------------------------------------------------------
// External files inclusions.
// --------------------------------------------------------------------------
#include <pic.h>                                                // Use a PIC16F877A.
#include "io.h"



// --------------------------------------------------------------------------
// This reads the current status of the numeric keypad by scanning all raws
// starting by row 1. It returns an ASCII character corresponding to the
// corresponding key ('0'..'9', '*' or '#'). If no key is pressed 0 is
// retourned.
// No debounce is performed and should be performed before calling.
// --------------------------------------------------------------------------
unsigned char read_keypad(void)
{
    unsigned char key_pressed;

    KEY_ROW_1 = 0;                                              // Start checking row 1.
    NOP();
    KEY_ROW_2 = 1;
    NOP();
    KEY_ROW_3 = 1;
    NOP();
    KEY_ROW_4 = 1;
    NOP();

    if (KEY_COL_1 == 0)                                         // Check if a key is pressed.
        key_pressed = '1';
    else if (KEY_COL_2 == 0)
        key_pressed = '2';
    else if (KEY_COL_3 == 0)
        key_pressed = '3';
    else
    {

        KEY_ROW_1 = 1;                                          // The key isn't in row 1, try row 2.
        NOP();
        KEY_ROW_2 = 0;
        NOP();
        KEY_ROW_3 = 1;
        NOP();
        KEY_ROW_4 = 1;
        NOP();
        
        if (KEY_COL_1 == 0)                                     // Check if a key is pressed.
            key_pressed = '4';
        else if (KEY_COL_2 == 0)
            key_pressed = '5';
        else if (KEY_COL_3 == 0)
            key_pressed = '6';
        else
        {

            KEY_ROW_1 = 1;                                      // The key isn't in row 2, try row 3.
            NOP();
            KEY_ROW_2 = 1;
            NOP();
            KEY_ROW_3 = 0;
            NOP();
            KEY_ROW_4 = 1;
            NOP();

            if (KEY_COL_1 == 0)                                 // Check if a key is pressed.
                key_pressed = '7';
            else if (KEY_COL_2 == 0)
                key_pressed = '8';
            else if (KEY_COL_3 == 0)
                key_pressed = '9';
            else
            {

                KEY_ROW_1 = 1;                                  // The key isn't in row 3, try row 4.
                NOP();
                KEY_ROW_2 = 1;
                NOP();
                KEY_ROW_3 = 1;
                NOP();
                KEY_ROW_4 = 0;
                NOP();

                if (KEY_COL_1 == 0)                             // Check if a key is pressed.
                    key_pressed = '*';
                else if (KEY_COL_2 == 0)
                    key_pressed = '0';
                else if (KEY_COL_3 == 0)
                    key_pressed = '#';
                else

                    key_pressed = 0;                            // No key is pressed.

            }
        }
    }

    KEY_ROW_1 = 0;                                              // Restore rows for any key detection.
    NOP();
    KEY_ROW_2 = 0;
    NOP();
    KEY_ROW_3 = 0;
    NOP();
    KEY_ROW_4 = 0;
    NOP();

    return key_pressed;                                         // Return pressed key.
}



// --------------------------------------------------------------------------
// This procedure initializes the I/O lines.
// --------------------------------------------------------------------------
void IO_init(void)
{
    PORTA = 0b00000000;                                         // Reset PORTA.
//              
//              +-- --> LCD_D4
//              +--- --> LCD_D5
//              +---- --> LCD_D6
//              +----- --> LCD_D7
//              +------ --> LCD_RS
//              +------- --> LCD_EN
    ADCON1 = 0x06;                                              // All digital.
    TRISA = 0b00000000;                                         // PORTA all output.

    PORTB = 0b00000000;                                         // Reset PORTB.
//            
//            +-- <-- KEY_COL_1
//            +--- <-- KEY_COL_2
//            +---- <-- KEY_COL_3
//            +----- <-- ENC_CHB
//            +------ <-- ENC_CHA
//            +------- <-- ENC_BTN
//            +-------- <-- BTN2
//            +--------- <-- BTN1
    RBPU = 0;                                                   // Enable weak pullup on PORTB.
    TRISB = 0b11111111;                                         // PORTB all input.

    PORTC = 0b00000000;                                         // Reset PORTC.
//            
//            +-- --> DDS_CS
//            +--- <-> DDS_UPDCLK
//            +---- --> DDS_IORST
//            +----- --> DDS_SCLK
//            +------ <-- DDS_SDI
//            +------- --> DDS_SDO
//            +-------- --> RS232_RXD
//            +--------- <-- RS232_TXD
    TRISC = 0b10010010;                                         // Set inputs/outputs (start with DDS_UPDCLK as input).

    PORTD = 0b00000000;                                         // Reset PORTD.
//            
//            +-- --> KEY_ROW_1                          // In order to detect keypress all 4 rows should stay low.
//            +--- --> KEY_ROW_2
//            +---- --> KEY_ROW_3
//            +----- --> KEY_ROW_4
//            +------     Reserved
//            +-------     Reserved
//            +-------- <-- RS232_RTS 
//            +--------- --> RS232_CTS
    TRISD = 0b01000000;                                         // Set inputs/outputs (reserved I/O are output)

    PORTE = 0b00000000;                                         // Reset PORTE.
//                 
//                 +-- --> DDS_RESET
//                 +---     Reserved
//                 +----     Reserved
    TRISE = 0b00000000;                                         // Set inputs/outputs (reserved I/O are output)
}


