#ifndef _IO_H
#define _IO_H



/*===========================================================================

I/O connections:
----------------

RA0  ---->  LCD_D4      LCD display data bit 4
RA1  ---->  LCD_D5      LCD display data bit 5
RA2  ---->  LCD_D6      LCD display data bit 6
RA3  ---->  LCD_D7      LCD display data bit 7
RA4  ---->  LCD_RS      LCD display Register Select line (0=command, 1=data)
RA5  ---->  LCD_EN      LCD display Enable line (1=enable)

RB0  <----  KBD_COL1    Matrix keyboard column 1 input (0=active)
RB1  <----  KBD_COL2    Matrix keyboard column 2 input (0=active)
RB2  <----  KBD_COL3    Matrix keyboard column 3 input (0=active)
RB3  <----  ENC_CHA     Encoder channel A
RB4  <----  ENC_CHB     Encoder channel B
RB5  <----  ENC_BTN     Encoder button (0=active)
RB6  <----  BTN2        P2 button (0=active)
RB7  <----  BTN1        P1 button (0=active)

RC0  ---->  DDS_CS
RC1  <--->  DDS_UPDCLK
RC2  ---->  DDS_IORST
RC3  ---->  DDS_SCLK
RC4  <----  DDS_SDI
RC5  ---->  DDS_SDO
RC6  ---->  RS232_RXD   RS_232 RXD line
RC7  <----  RS232_TXD   RS_232 TXD line

RD0  ---->  KBD_ROW1    Matrix keyboard row 1 output (0=active)
RD1  ---->  KBD_ROW2    Matrix keyboard row 2 output (0=active)
RD2  ---->  KBD_ROW3    Matrix keyboard row 3 output (0=active)
RD3  ---->  KBD_ROW4    Matrix keyboard row 4 output (0=active)
RD4  -----  Spare
RD5  -----  Spare
RD6  <----  RS232_RTS   RS_232 RTS line (inverted)
RD7  ---->  RS232_CTS   RS_232 CTS line (inverted)

RE0  ---->  DDS_RESET
RE1  -----  Spare
RE2  -----  Spare

============================================================================= */



// --------------------------------------------------------------------------
// I/O lines definition.
// --------------------------------------------------------------------------
#define LCD_D4          RA0     // LCD display data bit 4
#define LCD_D5          RA1     // LCD display data bit 5
#define LCD_D6          RA2     // LCD display data bit 6
#define LCD_D7          RA3     // LCD display data bit 7
#define LCD_RS          RA4     // LCD display Register Select line (0=command, 1=data)
#define LCD_EN          RA5     // LCD display Enable line (1=enable)
#define LCD_DATA        PORTA   // LCD data port (use only RA0..3)

#define KEY_COL_1       RB0     // Keypad column 1 (input)
#define KEY_COL_2       RB1     // Keypad column 2 (input)
#define KEY_COL_3       RB2     // Keypad column 3 (input)
#define ENC_CHA         RB3     // Encoder channel A
#define ENC_CHB         RB4     // Encoder channel B
#define ENC_BTN         RB5     // Encoder button (0=active)
#define BTN2            RB6     // P2 button (0=active)
#define BTN1            RB7     // P1 button (0=active)

#define DDS_CS          RC0     // AD9852 chip select
#define DDS_UPDCLK      RC1     // AD9852 update clock (bidirectional)
#define DDS_IORST       RC2     // AD9852 I/O reset
#define DDS_SCLK        RC3     // AD9852 serial clock
#define DDS_SDI         RC4     // AD9852 serial data input
#define DDS_SDO         RC5     // AD9852 serial data output
#define RS232_RXD       RC6     // RS232 RXD line (PIC output)
#define RS232_TXD       RC7     // RS232 TXD line (PIC input)
#define DDS_UPDCLK_TRIS TRISC1  // AD9852 update clock direction

#define KEY_ROW_1       RD0     // Keypad row 1 (output)
#define KEY_ROW_2       RD1     // Keypad row 2 (output)
#define KEY_ROW_3       RD2     // Keypad row 3 (output)
#define KEY_ROW_4       RD3     // Keypad row 4 (output)
#define RS232_RTS       RD6     // RS232 RTS line (PIC input)
#define RS232_CTS       RD7     // RS232 CTS line (PIC output)

#define DDS_RESET       RE0     // AD9852 master reset




// --------------------------------------------------------------------------
// Exported procedures and functions (see io.c for description).
// --------------------------------------------------------------------------
unsigned char read_keypad(void);
void IO_init(void);



#endif


